<?php
if(!defined('BASEPATH')){
	define('BASEPATH', dirname($_SERVER['SCRIPT_FILENAME']) . '/');
}
require_once('system/init.php');

require_once(BASEPATH . 'system/libs/input.php');
require_once(BASEPATH . 'ui/libs/ui.php');

$input	= new input();
$ui		= new ui();
$data	= array();

$segments = explode('/', $input->get('view'));

// Segment 1: Function
if(isset($segments[0])){
	$function = strtolower($segments[0]);
} else {
	$function = 'chat';
}

// Segment 2: Server
if(isset($segments[1])){
	$server_id = (int)$segments[1];
}

// Segment 3: Filter/Date
if(isset($segments[2]) && trim($segments[2])!=''){
	$date = strtotime($segments[2]);
} else {
	$date = mktime(0, 0, 0);
}

if(mktime(0, 0, 0)==$date){
	$logdate = date('Y-m-d');
} else {
	$logdate = date('Y-m-d', $date);
}

/* ============================================================
| Functions
*/
function encode_datetime($datetime){
	if(!is_numeric($datetime)){
		$datetime = strtotime($datetime);
	}

	#return date('c', $datetime);
	return date('Y-m-d H:i:s', $datetime);
}

/* ============================================================
| Load Logs Freshness
*/
$sql = "SELECT * FROM log_index ORDER BY datetime_parsed DESC LIMIT 1";
foreach($db->query($sql) as $row){
	$data['lastparse'] = strtotime($row['datetime_parsed']);
}

/* ============================================================
| Server
*/

// Load Server Meta
$sql = "SELECT * FROM server_meta";

foreach($db->query($sql) as $row){
	$server_meta[$row['server_id']][$row['field']] = $row['value'];
}

// Load Servers
$sql = "SELECT 
		server_id AS id,
		extension,
		name,
		mods
	FROM server_index
	WHERE status>=1
	ORDER BY name ASC
";

$servers = array();
foreach($db->query($sql) as $row){
	$data['servers'][] = $row;
	
	# Internal data
	if(isset($server_meta[$row['id']])){
		$row['metadata'] = $server_meta[$row['id']];
	} else {
		$row['metadata'] = array();
	}
	
	$servers[] = $row;
}

// Load Class Definitions
if(isset($data['servers'])){
	foreach($data['servers'] as $key=>$server){
		if(!isset($server['extension'])){
			$logger->log('Server config: Extension not specified');
		}

		$ext_name	= strtolower($server['extension']);
		$ext_path 	= BASEPATH . 'extensions/' . $server['extension'];

		require_once($ext_path . '/' . $server['extension'] . '.php');
		$classname = $ext_name;
		$servers[$key]['capabilities'] = $classname::$capabilities;
		$data['servers'][$key]['capabilities'] = $classname::$capabilities;
	}
}

// Load Active Server
if(isset($server_id) && is_numeric($server_id) && $server_id>0){
	$sql = "SELECT 
				server_id AS id,
				extension,
				name
			FROM server_index
			WHERE status>=1
			AND server_id=" . (int)$server_id . "
			ORDER BY name ASC
			LIMIT 1";
	
	foreach($db->query($sql) as $row){
		$server = $row;
	}
	
	if(isset($server)){
		if(!isset($server['extension'])){
			$logger->log('Server config: Extension not specified');
		}
	} else {
		$data['errors'][] = 'Could not load server "#' . $server_id . '"';
		$ui->render($data);
	}
}


/* ============================================================
|	Get Dataset
*/
$filename	= $segments[0];
$classname	= 'dataset_' . $segments[0];

require(BASEPATH . 'ui/models/dataset.php');
$filename = BASEPATH . 'ui/models/datasets/' . $filename . '.php';
if(file_exists($filename)){
	require($filename);

	$dataset = new $classname();
	$dataset->input = $input;
	if(isset($server_id)){
		$dataset->set('server', array('id' => $server_id));
	}
	if(isset($servers)){
		$dataset->set('servers', $servers);
	}
	if(isset($segments)){
		$dataset->set('segments', $segments);
	}
	$dataset->set_filters_by_segments($segments);
	$result = $dataset->get();
	#$data = array_merge($data, $result);
	$result['lastparse'] = $data['lastparse'];
	$ui->render($result);
} else {
	$data['errors'][] = 'Dataset not found';
}

/* ============================================================
| Done
*/
$ui->render($data);


