<?php
class Minecraft_mod_bungeecord{
	private $server			= array();			# Current server settings loaded
	
	function __construct($server=array()) {
		$this->server = $server;
	}
	
	function option($field, $default=false){
		if(isset($this->server['metadata'][$field])){
			return $this->server['metadata'][$field];
		} else if(isset($this->server[$field])){
			return $this->server[$field];
		}

		return $default;
	}
	
	
	/* ============================================================
	 * EXPORT START
	 * ------------------------------------------------------------
	 * Exporter Function to call at begining of export
	 */
	function export_start($server=null){

	}
	
	
	/* ============================================================
	 * CHAT
	 * ------------------------------------------------------------
	 * Used to parse each line of the log files
	 * Should return either false or a structure array
	 * false in the event of a non-match
	 * structure array depending on properties
	 */

	public $regexp_name		= '[A-z0-9_]{2,16}';
	public $regexp_ip		= '[0-9\.]+';
	public $regexp_coords	= '-?[0-9\.]+,\s*-?[0-9\.]+,\s*-?[0-9\.]+';
	
	private $appendable		= false;
	
	function parse_log_line($line){
		
		return false;
	}
	
	
	/* ============================================================
	 * EXPORT END
	 * ------------------------------------------------------------
	 * Exporter Function to call at end of export
	 */
	function export_end($server=null){
		if($server!==null){$this->server = $server;}
		
		// Add code here
	}
	
	
	/* ============================================================
	 * QUERY SERVER
	 * ------------------------------------------------------------
	 * Returns information via directly query to the server
	 */
	
	function query(){
		$host		= $this->option('host', '127.0.0.1');
		$version	= $this->option('version', '1.8.*');
		$port		= $this->option('port', 25565); 

        if (substr_count($host , '.') != 4) $host = gethostbyname($host);

        $returndata = array();
        $returndata['hostname']		= $host;
        $returndata['version']		= false;
        $returndata['protocol']		= false;
        $returndata['users']		= array(
			'max'		=> false,
			'online'	=> false,
		);
        $returndata['motd']			= false;
        $returndata['motd_raw']		= false;
        $returndata['favicon']		= false;
        $returndata['ping']			= -1;

        $socket = $this->connect($host, $port);
		
        if(!$socket) {
            return false;
        }
        
		// 1.7 or newer
        $start = microtime(true);

        $handshake = pack('cccca*', hexdec(strlen($host)), 0, 0x04, strlen($host), $host).pack('nc', $port, 0x01);

        socket_send($socket, $handshake, strlen($handshake), 0); //give the server a high five
        socket_send($socket, "\x01\x00", 2, 0);
        socket_read( $socket, 1 );

        $ping = round((microtime(true)-$start)*1000); //calculate the high five duration

        $packetlength = $this->read_packet_length($socket);

        if($packetlength < 1) {
            return false;
        }

        socket_read($socket, 1);

        $packetlength = $this->read_packet_length($socket);

        $data = socket_read($socket, $packetlength, PHP_NORMAL_READ);

        if(!$data) {
            return false;
        }
		
        $data = json_decode($data, true);
		foreach($data as $key=>$row){
			$serverdata[$key] = $row;
		}
		#echo '<pre> ' .print_r($serverdata, true) . '</pre>';

		if(is_array($data['description']) && isset($data['description']['text'])){
    	    $motd = $data['description']['text'];
        } else {
    	    $motd = $data['description'];
        }
        $serverdata['motd_raw'] = $motd;
        $motd = preg_replace("/(§.)/", "",$motd);
        $motd = preg_replace("/[^[:alnum:][:punct:] ]/", "", $motd);

		
		$returndata['users']['online']	= $serverdata['players']['online'];
		$returndata['users']['max']		= $serverdata['players']['max'];
		if(isset($serverdata['players']['sample'])){
			if(!function_exists('cmp_by_name')){
				function cmp_by_name($a, $b) {
					return $a["name"] > $b["name"];
				}
			}
			usort($serverdata['players']['sample'], "cmp_by_name");
			$returndata['users']['sample']	= $serverdata['players']['sample'];
		}
		
        $returndata['motd'] = $motd;
        $returndata['favicon'] = (isset($data['favicon']) ? $data['favicon'] : '');
        $returndata['ping'] = $ping;

		// Done
        $this->disconnect($socket);

        return $returndata;
    }

    private function connect($host, $port) {
    	$socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
    	$result = @socket_connect($socket, $host, $port); 
		if($result!==false){
    		return $socket;
		} else {
			return false;
		}
    }

    private function disconnect($socket) {
        if($socket != null) {
            socket_close($socket);
        }
    }

    private function read_packet_length($socket) {
        $a = 0;
        $b = 0;
        while(true) {
            $c = socket_read($socket, 1);
            if(!$c) {
                return 0;
            }
            $c = Ord($c);
            $a |= ($c & 0x7F) << $b++ * 7;
            if( $b > 5 ) {
                return false;
            }
            if(($c & 0x80) != 128) {
                break;
            }
        }
        return $a;
    }
	
}

