<?php
class Minecraft_mod_vanilla{
	private $server			= array();			# Current server settings loaded
	
	function __construct($server=array()) {
		$this->server = $server;
	}
	
	function option($field, $default=false){
		if(isset($this->server['metadata'][$field])){
			return $this->server['metadata'][$field];
		} else if(isset($this->server[$field])){
			return $this->server[$field];
		}

		return $default;
	}
	
	
	/* ============================================================
	 * EXPORT START
	 * ------------------------------------------------------------
	 * Exporter Function to call at begining of export
	 */
	function export_start($server=null){
		if($server!==null){$this->server = $server;}
		
		$dir 	= rtrim($this->server['directory'], '/');
		$datetime_updates = date('Y-m-d H:i:s');

		# File: Banned Players
		$meta = 'banned-players';
		$filename = $dir . '/' . $meta . '.json';
		if(file_exists($filename)){
			$contents = file_get_contents($filename);
			$rows = json_decode($contents, true);
			#echo '<pre>' . print_r($rows, true) . '</pre>'; exit;

			$this->db->delete('player_meta', array(
				'server_id'			=> $this->server['id'],
				'meta'				=> $meta,
			));
			
			foreach($rows as $row){
				$this->db->insert_update('player_meta', array(
					'server_id'			=> $this->server['id'],
					'datetime_updated'	=> $datetime_updates,
					'meta'				=> $meta,
					'user_name'			=> $row['name'],
					'date1'				=> date('Y-m-d H:i:s', strtotime($row['created'])),
					'value1'			=> $row['source'],
					'value2'			=> $row['expires'],
					'value3'			=> $row['reason'],
				), array(
					'datetime_updated'	=> true
				));
			}
		}

		# File: Banned IPs
		$meta = 'banned-ips';
		$filename = $dir . '/' . $meta . '.json';
		if(file_exists($filename)){
			$contents = file_get_contents($filename);
			$rows = json_decode($contents, true);
			#echo '<pre>' . print_r($rows, true) . '</pre>'; exit;

			$this->db->delete('player_meta', array(
				'server_id'			=> $this->server['id'],
				'meta'				=> $meta,
			));
			
			foreach($rows as $row){	
				$this->db->insert_update('player_meta', array(
					'server_id'			=> $this->server['id'],
					'datetime_updated'	=> $datetime_updates,
					'meta'				=> $meta,
					'user_name'			=> $row['ip'],
					'date1'				=> date('Y-m-d H:i:s', strtotime($row['created'])),
					'value1'			=> $row['source'],
					'value2'			=> $row['expires'],
					'value3'			=> $row['reason'],
					'value4'			=> $row['ip'],
				), array(
					'datetime_updated'	=> true
				));
			}
		}
		#TODO use logs parsed to update user_name

		# File: Ops
		$meta = 'ops';
		$filename = $dir . '/' . $meta . '.json';
		if(file_exists($filename)){
			$contents = file_get_contents($filename);
			$rows = json_decode($contents, true);
			#echo '<pre>' . print_r($rows, true) . '</pre>'; exit;

			$this->db->delete('player_meta', array(
				'server_id'			=> $this->server['id'],
				'meta'				=> $meta,
			));
			
			foreach($rows as $row){	
				$this->db->insert_update('player_meta', array(
					'server_id'			=> $this->server['id'],
					'datetime_updated'	=> $datetime_updates,
					'meta'				=> $meta,
					'user_name'			=> $row['name'],
					'value1'			=> $row['level'],
				), array(
					'datetime_updated'	=> true
				));
			}
		}

		# File: Whitelist
		$meta = 'whitelist';
		$filename = $dir . '/' . $meta . '.json';
		if(file_exists($filename)){
			$contents = file_get_contents($filename);
			$rows = json_decode($contents, true);
			#echo '<pre>' . print_r($rows, true) . '</pre>'; exit;

			$this->db->delete('player_meta', array(
				'server_id'			=> $this->server['id'],
				'meta'				=> $meta,
			));
			
			foreach($rows as $row){	
				$this->db->insert_update('player_meta', array(
					'server_id'			=> $this->server['id'],
					'datetime_updated'	=> $datetime_updates,
					'meta'				=> $meta,
					'user_name'			=> $row['name'],
				), array(
					'datetime_updated'	=> true
				));
			}
		}

		# Drop Records not updated
		$sth = $this->db->conn->prepare("DELETE FROM player_meta WHERE server_id=:server_id AND datetime_updated<:datetime_updated");
		$sth->execute(array(':server_id' => $this->server['id'], ':datetime_updated' => $datetime_updates));
	}
	
	
	/* ============================================================
	 * CHAT
	 * ------------------------------------------------------------
	 * Used to parse each line of the log files
	 * Should return either false or a structure array
	 * false in the event of a non-match
	 * structure array depending on properties
	 */

	public $regexp_name		= '[A-z0-9_]{2,16}';
	public $regexp_ip		= '[0-9\.]+';
	public $regexp_coords	= '-?[0-9\.]+,\s*-?[0-9\.]+,\s*-?[0-9\.]+';
	
	private $appendable		= false;
	
	function parse_log_line($line){

		if(preg_match('/\[([0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2})\]\s+\[([^\]]+)\]:\s+(.*?)$/i', $line, $matches)){
			$this->appendable = false;


			$time	= trim($matches[1]);
			$type	= strtolower(trim($matches[2]));
			$text	= trim($matches[3]);

			# Log in information
			# UUID of player playername is 82b39f46-xxxx-xxxx-xxxx-e36cbbf5e241
			if(stripos($text, 'UUID of player')!==false && preg_match('/UUID of player (\S+) is (\S+)/is', $text, $breakdown)){
				#echo '<pre>' . print_r($matches, true) . '</pre>';
				#echo '<pre>' . print_r($breakdown, true) . '</pre>';
				
				return array(
					'time'	=> $time,
					'type'		=> 'uuid',
					'user'		=> trim(trim($breakdown[1])),
					'text'		=> $breakdown[2],
				);
			} 

			# Lag 
			else if(		stripos($text, 'Can\'t keep up! Did the system time change, or is the server overloaded?')!==false	# (Vanilla 1.8)
				||	stripos($text, '--- Server Lag: ')!==false # (Rainbow Project)
			){
				#Can't keep up! Did the system time change, or is the server overloaded? Running 2040ms behind, skipping 40 tick(s)
				preg_match('/Running ([0-9]+)ms behind, skipping ([0-9]+) tick/is', $text, $breakdown);

				return array(
					'time'	=> $time,
					'type'		=> 'lag',
					'text'		=> json_encode(array(
						'text'		=> $text,
						'latency'	=> $breakdown[1],
						'ticks'		=> $breakdown[2],
					))
				);
			} 
			
			# Login IP and Coords
			else if(stripos($text, 'logged in with entity id')!==false){
				# aamche[/202.180.86.136:34280] logged in with entity id 1437768 at ([world]602.2744076595088, 64.0, -39.02937949785859)
			
				preg_match('/(.*?)\[\/?(.*?):[0-9]+\] logged in with entity id [0-9]+ at \((\[([A-z0-9_-]+)\])?\s*(-?[0-9\.]+),\s*(-?[0-9\.]+),\s*(-?[0-9\.]+)\)/is', $text, $breakdown);
				
				#echo '<pre>' . print_r($matches, true) . '</pre>';
				#echo '<pre>' . print_r($breakdown, true) . '</pre>';exit;
			
				return array(
					'time'		=> $time,
					'type'		=> 'joined',
					'user'		=> trim($breakdown[1]),
					'ip'		=> trim(trim($breakdown[2])),
					'coords'	=> array(
									'dimension'	=> (isset($breakdown[4]) ? $breakdown[4] : ''),
									'x'	=> (isset($breakdown[5]) ? round($breakdown[5]) : ''),
									'y'	=> (isset($breakdown[6]) ? round($breakdown[6]) : ''),
									'z'	=> (isset($breakdown[7]) ? round($breakdown[7]) : ''),
								),
					'text'		=> $text,
				);
			}

			# Join/Leave
			else if(stripos($text, 'joined the game')!==false || stripos($text, 'left the game')!==false){
				preg_match('/(.*?)(joined|left) the game/is', $text, $breakdown);
				
				#echo '<pre>' . print_r($matches, true) . '</pre>';
				#echo '<pre>' . print_r($breakdown, true) . '</pre>';

				$action	= strtolower(trim($breakdown[2]));
			
				return array(
					'time'		=> $time,
					'type'		=> ($action=='left' ? 'left' : 'joined'),
					'user'		=> trim($breakdown[1]),
					'text'		=> $text,
				);
			}

			# Chat
			else if($type=='server thread/info' && preg_match('/<([^>]+)>\s+(.*?)$/is', $text, $breakdown)){
				#echo '<pre>' . print_r($breakdown, true) . '</pre>';
				return array(
					'time'		=> $time,
					'type'		=> 'chat',
					'user'		=> $breakdown[1],
					'text'		=> $breakdown[2],
				);
			} 
			
			# Server Command
			# [00:02:59] [Server thread/INFO]: [VanJeans: Teleported VanJeans to CountBarkula]
			else if($type=='server thread/info' && preg_match('/\[([A-z0-9_]+):\s+(.*?)]$/is', $text, $breakdown)){
				return array(
					'time'		=> $time,
					'type'		=> 'command',
					'user'		=> $breakdown[1],
					'text'		=> $breakdown[2],
				);
			} 
			
			# Error
			else if($type=='server thread/error'){
				$this->appendable = true;
				return array(
					'time'			=> $time,
					'type'			=> 'error',
					'text'			=> $text,
					'appendable'	=> true,
				);
			} else {
				#echo '<pre>' . print_r($matches, true) . '</pre>';
			}

			# Unknown
			#return 'unknown: ' . print_r($matches, true);
		}
		// Append Java Error
		else if ($this->appendable===true){
			return array(
				'type'		=> 'append',
				'text'		=> $line,
			);
		}
		// Unknown line format
		else {
			#return 'line: ' . $line;
		}
		
		return false;
	}
	
	
	/* ============================================================
	 * EXPORT END
	 * ------------------------------------------------------------
	 * Exporter Function to call at end of export
	 */
	function export_end($server=null){
		if($server!==null){$this->server = $server;}
		
		// Add code here
	}
	
	
	/* ============================================================
	 * QUERY SERVER
	 * ------------------------------------------------------------
	 * Returns information via directly query to the server
	 */
	
	function query(){
		$host		= $this->option('host', '127.0.0.1');
		$version	= $this->option('version', '1.8.*');
		$port		= $this->option('port', 25565); 

        if (substr_count($host , '.') != 4) $host = gethostbyname($host);

        $returndata = array();
        $returndata['hostname']		= $host;
        $returndata['version']		= false;
        $returndata['protocol']		= false;
        $returndata['users']		= array(
			'max'		=> false,
			'online'	=> false,
		);
        $returndata['motd']			= false;
        $returndata['motd_raw']		= false;
        $returndata['favicon']		= false;
        $returndata['ping']			= -1;

        $socket = $this->connect($host, $port);
		
        if(!$socket) {
            return false;
        }
        
        if(preg_match('/1\.5|1\.6/', $version)) {

            $start = microtime(true);

            socket_send($socket, "\xFE\x01", 2, 0);
            $length = socket_recv($socket, $data, 512, 0);

            $ping = round((microtime(true)-$start)*1000);//calculate the high five duration
            
            if($length < 4 || $data[0] != "\xFF") {
                return false;
            }

            $motd = "";
            $motdraw = "";

            //Evaluate the received data
            if (substr((string)$data, 3, 5) == "\x00\xa7\x00\x31\x00"){

                $result = explode("\x00", mb_convert_encoding(substr((string)$data, 15), 'UTF-8', 'UCS-2'));
                $motd = $result[1];
                $motdraw = $motd;

            } else {

            	$result = explode('§', mb_convert_encoding(substr((string)$data, 3), 'UTF-8', 'UCS-2'));
                foreach ($result as $key => $string) {
                    if($key != sizeof($result)-1 && $key != sizeof($result)-2 && $key != 0) {
                        $motd .= '§'.$string;
                    }
                }
                $motdraw = $motd;
            }

            $motd = preg_replace("/(§.)/", "", $motd);
            $motd = preg_replace("/[^[:alnum:][:punct:] ]/", "", $motd); //Remove all special characters from a string

            $returndata['version'] = $result[0];
            $returndata['users'] = $result[sizeof($result)-2];
            $returndata['maxplayers'] = $result[sizeof($result)-1];
            $returndata['motd'] = $motd;
            $returndata['motd_raw'] = $motdraw;
            $returndata['ping'] = $ping;

        } 
		# 1.7 or newer
		else {
            $start = microtime(true);

            $handshake = pack('cccca*', hexdec(strlen($host)), 0, 0x04, strlen($host), $host).pack('nc', $port, 0x01);

            socket_send($socket, $handshake, strlen($handshake), 0); //give the server a high five
            socket_send($socket, "\x01\x00", 2, 0);
            socket_read( $socket, 1 );

            $ping = round((microtime(true)-$start)*1000); //calculate the high five duration

            $packetlength = $this->read_packet_length($socket);

            if($packetlength < 1) {
                return false;
            }

            socket_read($socket, 1);

            $packetlength = $this->read_packet_length($socket);

            $data = socket_read($socket, $packetlength, PHP_NORMAL_READ);

            if(!$data) {
                return false;
            }
			
            $data = json_decode($data, true);
			foreach($data as $key=>$row){
				$serverdata[$key] = $row;
			}
			#echo '<pre> ' .print_r($serverdata, true) . '</pre>';

			if(is_array($data['description']) && isset($data['description']['text'])){
        	    $motd = $data['description']['text'];
            } else {
        	    $motd = $data['description'];
            }
            $serverdata['motd_raw'] = $motd;
            $motd = preg_replace("/(§.)/", "",$motd);
            $motd = preg_replace("/[^[:alnum:][:punct:] ]/", "", $motd);

			
			$returndata['users']['online']	= $serverdata['players']['online'];
			$returndata['users']['max']		= $serverdata['players']['max'];
			if(isset($serverdata['players']['sample'])){
				if(!function_exists('cmp_by_name')){
					function cmp_by_name($a, $b) {
						return $a["name"] > $b["name"];
					}
				}
				usort($serverdata['players']['sample'], "cmp_by_name");
				$returndata['users']['sample']	= $serverdata['players']['sample'];
			}
			
            $returndata['motd'] = $motd;
            $returndata['favicon'] = (isset($data['favicon']) ? $data['favicon'] : '');
            $returndata['ping'] = $ping;
        }

        $this->disconnect($socket);

        return $returndata;

    }

    private function connect($host, $port) {
    	$socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
    	$result = @socket_connect($socket, $host, $port); 
		if($result!==false){
    		return $socket;
		} else {
			return false;
		}
    }

    private function disconnect($socket) {
        if($socket != null) {
            socket_close($socket);
        }
    }

    private function read_packet_length($socket) {
        $a = 0;
        $b = 0;
        while(true) {
            $c = socket_read($socket, 1);
            if(!$c) {
                return 0;
            }
            $c = Ord($c);
            $a |= ($c & 0x7F) << $b++ * 7;
            if( $b > 5 ) {
                return false;
            }
            if(($c & 0x80) != 128) {
                break;
            }
        }
        return $a;
    }
	
}

