<?php
/*
 * Venture Chat
 * Bukkit/Spigot Plugin
 * http://dev.bukkit.org/bukkit-plugins/venturechat/
 * https://www.spigotmc.org/resources/venturechat.771/
 */
class Minecraft_mod_venturechat{
	private $server			= array();			# Current server settings loaded
	
	function __construct($server=array()) {
		$this->server = $server;
	}
	
	function option($field, $default=false){
		if(isset($this->server['metadata'][$field])){
			return $this->server['metadata'][$field];
		} else if(isset($this->server[$field])){
			return $this->server[$field];
		}

		return $default;
	}
		
	/* ============================================================
	 * CHAT
	 * ------------------------------------------------------------
	 * Used to parse each line of the log files
	 * Should return either false or a structure array
	 * false in the event of a non-match
	 * structure array depending on properties
	 */

	public $regexp_name		= '[A-z0-9_]{2,16}';
	public $regexp_ip		= '[0-9\.]+';
	public $regexp_coords	= '-?[0-9\.]+,\s*-?[0-9\.]+,\s*-?[0-9\.]+';
	
	private $appendable		= false;
	
	function parse_log_line($line){
		if(preg_match('/\[([0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2})\]\s+\[([^\]]+)\]:\s+(.*?)$/i', $line, $matches)){
			$this->appendable = false;

			$time	= trim($matches[1]);
			$type	= strtolower(trim($matches[2]));
			$text	= trim($matches[3]);


			
			# Minecraft 1.8
			# Chat: Mineversechat or is this just spigot
			# MC 1.9ish
			# [10:19:59] [Async Chat Thread - #0/INFO]: [0;37;1m[[0;32;22mGlobal[0;37;1m] FuriousNinja2207[0;32;22m:[0;32;22m hi[m
			if(stripos($type, 'async chat thread')!==false && stripos($text, '[0;37;1m')!==false){
				#$text = str_replace('[0;37;1m', '', substr($text,0, -3));
				$text = preg_replace('/\[[0-9]+;[0-9]+;[0-9]+m\s*/is', '', $text);
				$text = preg_replace('/^.?[^\]]+\]\s*/is', '', trim($text));
				$text = preg_replace('/\[m$/is', '', $text);
				if(strpos($text, ':')!==false){
					list($user, $text) = explode(':', $text, 2);
					return array(
						'time'		=> $time,
						'type'		=> 'chat',
						'user'		=> trim($user),
						'text'		=> trim($text),
					);
				}
			}
			
			# Minecraft 1.8
			# Chat: Mineversechat or is this just spigot
			# [19:09:26] [Server thread/INFO]: [0;37;1m[Main] aamche[0;37;1m:[0;37;1m[0;37;1m warp speed[m
			# [16:36:13] [Server thread/INFO]: [0;37;1m[[0;32;22mGlobal[0;37;1m] [0;32;1m aamche[0;37;1m:[0;37;1m howdy all[m
			# [07:43:37] [Server thread/INFO]: [0;37;1m[[0;32;22mGlobal[0;37;1m] [[0;32;1mStaff§g] jamie109787[0;32;22m:[0;32;22m block torch block redstone[m
			# [08:47:49] [Server thread/INFO]: [0;37;1m[[0;36;1mHelp[0;37;1m] jamie109787[0;36;1m:[0;36;1m hi[m
			else if($type=='server thread/info' && stripos($text, '[0;37;1m')!==false){
				preg_match('/([A-z0-9_-]+)\[0;[0-9]+;[0-9]+m:\[0;[0-9]+;[0-9]+m([^]+)/is', $text, $text_matches);
				#echo '<pre>' . print_r($text_matches, true) . '</pre>';
				#exit;
				#$text = str_replace('[0;37;1m', '', substr($text,0, -3));
				#$text = preg_replace('/\[[0-9]+;[0-9]+;[0-9]+m\s*/is', '', $text);
				#$text = preg_replace('/^.?[^\]]+\]\s*/is', '', trim($text));
				#$text = preg_replace('/\[m$/is', '', $text);						# Remove trailing delimiter characters
				
				if(isset($text_matches[1]) && trim($text_matches[1])!=''){
					return array(
						'time'		=> $time,
						'type'		=> 'chat',
						'user'		=> trim($text_matches[1]),
						'text'		=> trim($text_matches[2]),
					);
				} else {
					echo "\n" . $text . "\n";
					echo '<pre>' . print_r($text_matches, true) . '</pre>';
				}
			}
		}
		
		return false;
	}
}
