<?php
class Teamspeak_mod_default{
	private $server			= array();			# Current server settings loaded
	private $query_conn		= array();
	
	function __construct($server=array()) {
		$this->server = $server;
	}
	
	function option($field, $default=false){
		if(isset($this->server['metadata'][$field])){
			return $this->server['metadata'][$field];
		} else if(isset($this->server[$field])){
			return $this->server[$field];
		}

		return $default;
	}
	
	
	/* ============================================================
	 * EXPORT START
	 * ------------------------------------------------------------
	 * Exporter Function to call at begining of export
	 */
	function export_start($server=null){
		if($server!==null){$this->server = $server;}
		return;
		$dir 	= rtrim($this->server['directory'], '/');
		$datetime_updates = date('Y-m-d H:i:s');

		# File: Banned Players
		$meta = 'banned-players';
		$filename = $dir . '/' . $meta . '.json';
		if(file_exists($filename)){
			$contents = file_get_contents($filename);
			$rows = json_decode($contents, true);
			#echo '<pre>' . print_r($rows, true) . '</pre>'; exit;

			$this->db->delete('player_meta', array(
				'server_id'			=> $this->server['id'],
				'meta'				=> $meta,
			));
			
			foreach($rows as $row){
				$this->db->insert_update('player_meta', array(
					'server_id'			=> $this->server['id'],
					'datetime_updated'	=> $datetime_updates,
					'meta'				=> $meta,
					'user_name'			=> $row['name'],
					'date1'				=> date('Y-m-d H:i:s', strtotime($row['created'])),
					'value1'			=> $row['source'],
					'value2'			=> $row['expires'],
					'value3'			=> $row['reason'],
				), array(
					'datetime_updated'	=> true
				));
			}
		}

		# File: Banned IPs
		$meta = 'banned-ips';
		$filename = $dir . '/' . $meta . '.json';
		if(file_exists($filename)){
			$contents = file_get_contents($filename);
			$rows = json_decode($contents, true);
			#echo '<pre>' . print_r($rows, true) . '</pre>'; exit;

			$this->db->delete('player_meta', array(
				'server_id'			=> $this->server['id'],
				'meta'				=> $meta,
			));
			
			foreach($rows as $row){	
				$this->db->insert_update('player_meta', array(
					'server_id'			=> $this->server['id'],
					'datetime_updated'	=> $datetime_updates,
					'meta'				=> $meta,
					'user_name'			=> $row['ip'],
					'date1'				=> date('Y-m-d H:i:s', strtotime($row['created'])),
					'value1'			=> $row['source'],
					'value2'			=> $row['expires'],
					'value3'			=> $row['reason'],
					'value4'			=> $row['ip'],
				), array(
					'datetime_updated'	=> true
				));
			}
		}
		#TODO use logs parsed to update user_name

		# File: Ops
		$meta = 'ops';
		$filename = $dir . '/' . $meta . '.json';
		if(file_exists($filename)){
			$contents = file_get_contents($filename);
			$rows = json_decode($contents, true);
			#echo '<pre>' . print_r($rows, true) . '</pre>'; exit;

			$this->db->delete('player_meta', array(
				'server_id'			=> $this->server['id'],
				'meta'				=> $meta,
			));
			
			foreach($rows as $row){	
				$this->db->insert_update('player_meta', array(
					'server_id'			=> $this->server['id'],
					'datetime_updated'	=> $datetime_updates,
					'meta'				=> $meta,
					'user_name'			=> $row['name'],
					'value1'			=> $row['level'],
				), array(
					'datetime_updated'	=> true
				));
			}
		}

		# File: Whitelist
		$meta = 'whitelist';
		$filename = $dir . '/' . $meta . '.json';
		if(file_exists($filename)){
			$contents = file_get_contents($filename);
			$rows = json_decode($contents, true);
			#echo '<pre>' . print_r($rows, true) . '</pre>'; exit;

			$this->db->delete('player_meta', array(
				'server_id'			=> $this->server['id'],
				'meta'				=> $meta,
			));
			
			foreach($rows as $row){	
				$this->db->insert_update('player_meta', array(
					'server_id'			=> $this->server['id'],
					'datetime_updated'	=> $datetime_updates,
					'meta'				=> $meta,
					'user_name'			=> $row['name'],
				), array(
					'datetime_updated'	=> true
				));
			}
		}

		# Drop Records not updated
		$sth = $this->db->conn->prepare("SELECT * FROM log_index WHERE server_id=:server_id AND datetime_updated<:datetime_updated");
		$sth->execute(array(':server_id' => $this->server['id'], ':datetime_updated' => $datetime_updates));
	}
	
	
	/* ============================================================
	 * CHAT
	 * ------------------------------------------------------------
	 * Used to parse each line of the log files
	 * Should return either false or a structure array
	 * false in the event of a non-match
	 * structure array depending on properties
	 */

	public $regexp_name		= '[A-z0-9_]{2,16}';
	public $regexp_ip		= '[0-9\.]+';
	public $regexp_coords	= '-?[0-9\.]+,\s*-?[0-9\.]+,\s*-?[0-9\.]+';
	
	private $appendable		= false;
	
	function parse_log_line($line){
		return;
		if(preg_match('/\[([0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2})\]\s+\[([^\]]+)\]:\s+(.*?)$/i', $line, $matches)){
			$this->appendable = false;


			$time	= trim($matches[1]);
			$type	= strtolower(trim($matches[2]));
			$text	= trim($matches[3]);

			# Log in information
			# UUID of player playername is 82b39f46-xxxx-xxxx-xxxx-e36cbbf5e241
			if(stripos($text, 'UUID of player')!==false && preg_match('/UUID of player (\S+) is (\S+)/is', $text, $breakdown)){
				#echo '<pre>' . print_r($matches, true) . '</pre>';
				#echo '<pre>' . print_r($breakdown, true) . '</pre>';
				
				return array(
					'time'	=> $time,
					'type'		=> 'uuid',
					'user'		=> trim(trim($breakdown[1])),
					'text'		=> $breakdown[2],
				);
			} 

			# Lag 
			else  if(		stripos($text, 'Can\'t keep up! Did the system time change, or is the server overloaded?')!==false	# (Vanilla 1.8)
				||	stripos($text, '--- Server Lag: ')!==false # (Rainbow Project)
			){
				#Can't keep up! Did the system time change, or is the server overloaded? Running 2040ms behind, skipping 40 tick(s)
				preg_match('/Running ([0-9]+)ms behind, skipping ([0-9]+) tick/is', $text, $breakdown);

				return array(
					'time'	=> $time,
					'type'		=> 'lag',
					'text'		=> json_encode(array(
						'text'		=> $text,
						'latency'	=> $breakdown[1],
						'ticks'		=> $breakdown[2],
					))
				);
			} 
			
			# Login IP and Coords
			else  if(stripos($text, 'logged in with entity id')!==false){
				# aamche[/202.180.86.136:34280] logged in with entity id 1437768 at ([world]602.2744076595088, 64.0, -39.02937949785859)
			
				preg_match('/(.*?)\[\/?(.*?):[0-9]+\] logged in with entity id [0-9]+ at \((\[([A-z0-9_-]+)\])?\s*(-?[0-9\.]+),\s*(-?[0-9\.]+),\s*(-?[0-9\.]+)\)/is', $text, $breakdown);
				
				#echo '<pre>' . print_r($matches, true) . '</pre>';
				#echo '<pre>' . print_r($breakdown, true) . '</pre>';exit;
			
				return array(
					'time'		=> $time,
					'type'		=> 'joined',
					'user'		=> trim($breakdown[1]),
					'ip'		=> trim(trim($breakdown[2])),
					'coords'	=> array(
									'dimension'	=> (isset($breakdown[4]) ? $breakdown[4] : ''),
									'x'	=> (isset($breakdown[5]) ? round($breakdown[5]) : ''),
									'y'	=> (isset($breakdown[6]) ? round($breakdown[6]) : ''),
									'z'	=> (isset($breakdown[7]) ? round($breakdown[7]) : ''),
								),
					'text'		=> $text,
				);
			}

			# Join/Leave
			else  if(stripos($text, 'joined the game')!==false || stripos($text, 'left the game')!==false){
				preg_match('/(.*?)(joined|left) the game/is', $text, $breakdown);
				
				#echo '<pre>' . print_r($matches, true) . '</pre>';
				#echo '<pre>' . print_r($breakdown, true) . '</pre>';

				$action	= strtolower(trim($breakdown[2]));
			
				return array(
					'time'		=> $time,
					'type'		=> ($action=='left' ? 'left' : 'joined'),
					'user'		=> trim($breakdown[1]),
					'text'		=> $text,
				);
			}

			# Chat
			else  if($type=='server thread/info' && preg_match('/<([^>]+)>\s+(.*?)$/is', $text, $breakdown)){
				return array(
					'time'		=> $time,
					'type'		=> 'chat',
					'user'		=> $breakdown[1],
					'text'		=> $breakdown[2],
				);
			} 
			
			# Server Command
			# [00:02:59] [Server thread/INFO]: [VanJeans: Teleported VanJeans to CountBarkula]
			else  if($type=='server thread/info' && preg_match('/\[([A-z0-9_]+):\s+(.*?)]$/is', $text, $breakdown)){
				return array(
					'time'		=> $time,
					'type'		=> 'command',
					'user'		=> $breakdown[1],
					'text'		=> $breakdown[2],
				);
			} 
			
			# Error
			else  if($type=='server thread/error'){
				$this->appendable = true;
				return array(
					'time'			=> $time,
					'type'			=> 'error',
					'text'			=> $text,
					'appendable'	=> true,
				);
			}

			# Unknown
			#return 'unknown: ' . print_r($matches, true);
		}
		// Append Java Error
		else if ($this->appendable===true){
			return array(
				'type'		=> 'append',
				'text'		=> $line,
			);
		}
		// Unknown line format
		else {
			#return 'line: ' . $line;
		}
		
		return false;
	}
	
	
	/* ============================================================
	 * EXPORT END
	 * ------------------------------------------------------------
	 * Exporter Function to call at end of export
	 */
	function export_end($server=null){
		if($server!==null){$this->server = $server;}
		
		// Add code here
	}
	
	
	/* ============================================================
	 * QUERY SERVER
	 * ------------------------------------------------------------
	 * Returns information via directly query to the server
	 */
	
	function query(){
		$host		= $this->option('host', '127.0.0.1');
		#$version	= $this->option('version', '1.8.*');
		$port		= $this->option('port', 10011); 
		$username	= $this->option('username', false); 
		$password	= $this->option('password', false); 

		if (substr_count($host , '.') != 4) {$host = gethostbyname($host);}

        $serverdata = array();
        $serverdata['hostname']		= $host;
        #$serverdata['version']		= false;
        $serverdata['protocol']		= false;
        $serverdata['users']		= array(
			#'max'		=> false,
			'online'	=> array(),
		);
        $serverdata['motd']			= false;
        $serverdata['motd_raw']		= false;
        $serverdata['favicon']		= false;
        $serverdata['ping']			= -1;
		
		if($username===false || $password===false){
			return array(
				'result'	=> 'error',
				'message'	=> 'Username and password required',
			);
		}
		
		$start = microtime(true);
        $socket = $this->connect($host, $port);
		
        if(!$socket) {
			return array(
				'result'	=> 'error',
				'message'	=> 'Unable to connect',
			);
        }
		
		
		#$data = $this->fetch('help');
		#echo '<pre>' . print_r($data, true) . '</pre>';
		
		$data = $this->fetch('login ' . $username . ' ' . $password, 'error id=');
		#echo '<pre>' . print_r($data, true) . '</pre>';
		$ping = microtime(true) - $start;
		
		$data = $this->fetch('use 1');
		#echo '<pre>' . print_r($data, true) . '</pre>';
		
		$list = $this->fetch('clientlist');
        if($list===false) {
			return array(
				'result'	=> 'error',
				'message'	=> 'Communication error, try again soon',
			);
        }
		$list = explode('|', $list);
		foreach($list as $key=>$row){
			$list[$key] = $this->decode_response_line($row);
		}
		
		foreach($list as $row){
			if(isset($row['client_nickname']) && isset($row['client_type']) && $row['client_type']==0){
				$serverdata['users']['sample'][] = array(
							'name'	=> $row['client_nickname']
						);
			}
		}
		if(isset($serverdata['users']['sample'])){
			$serverdata['users']['online'] = count($serverdata['users']['sample']);
			
			if(!function_exists('cmp_by_name')){
				function cmp_by_name($a, $b) {
					return $a["name"] > $b["name"];
				}
			}
			usort($serverdata['users']['sample'], "cmp_by_name");
		} else {
			$serverdata['users']['online'] = 0;
		}
		
		#echo '<pre>' . print_r($list, true) . '</pre>';
		
		$data = $this->fetch('help');
		#echo '<pre>' . print_r($data, true) . '</pre>';
		
		$serverinfo = $this->fetch('serverinfo');
		$serverinfo = $this->decode_response_line($serverinfo);
		#echo '<pre>' . print_r($serverinfo, true) . '</pre>';
        if($serverinfo===false) {
			return array(
				'result'	=> 'error',
				'message'	=> 'Communication error, try again soon',
			);
        }
		
		$data = $this->fetch('logout');
		#echo '<pre>' . print_r($data, true) . '</pre>';
		
		
		/*
		$serverdata['version'] = $data->version->name;
		$serverdata['protocol'] = $data->version->protocol;
		$serverdata['users'] = $data->players->online;
		$serverdata['maxplayers'] = $data->players->max;
		*/


		if(isset($serverinfo['virtualserver_name'])){
			$serverdata['motd'] = $serverinfo['virtualserver_name'];
		}
		
		#$serverdata['favicon'] = (isset($data['favicon']) ? $data['favicon'] : '');
		$serverdata['ping'] = (int)round($ping);
			
		// Done
        $this->disconnect();

        return $serverdata;

    }

	function connect($host, $port){ 
        if ($this->query_conn = @fsockopen($host, $port, $errno, $errstr, 1)) {  
            return true; 
        } else { 
            return false; 
        } 
    }
    function fetch($query=''){ 
        if(!@fwrite($this->query_conn, $query . " \r\n")>0){
			return false;
		}
		
		$data = '';
		while(!feof($this->query_conn)) {
			$c = fgetc($this->query_conn);
			if($c === false) break;
			$data .= $c;
			if(substr($data, -6)=='msg=ok') break;
			if(preg_match('/msg=[^\s]+[\s\n\r\t]/i', $data)) break;
		}
        return $data; 
    } 
    function disconnect(){ 
        $this->fetch('quit'); 
        fclose($this->query_conn); 
    }
	
	function decode_response_line($row){
		$return = array();
		$row = explode(' ', trim($row));
		foreach($row as $r){
			if(strpos($r, '=')!==false){
				list($k, $r) = explode('=', $r, 2);
			} else {
				$k = $r;
				$r = true;
			}
			$return[$k] = str_replace('\s', ' ', $r);
		}
		
		return $return;
	}
}

