<?php
class Terraria_mod_default{
	private $server			= array();			# Current server settings loaded
	private $query_conn		= array();
	
	function __construct($server=array()) {
		$this->server = $server;
	}
	
	function option($field, $default=false){
		if(isset($this->server['metadata'][$field])){
			return $this->server['metadata'][$field];
		} else if(isset($this->server[$field])){
			return $this->server[$field];
		}

		return $default;
	}
	
	
	/* ============================================================
	 * EXPORT START
	 * ------------------------------------------------------------
	 * Exporter Function to call at begining of export
	 */
	function export_start($server=null){
		if($server!==null){$this->server = $server;}
		
		// Add code here
	}
	
	
	/* ============================================================
	 * CHAT
	 * ------------------------------------------------------------
	 * Used to parse each line of the log files
	 * Should return either false or a structure array
	 * false in the event of a non-match
	 * structure array depending on properties
	 */

	public $regexp_name		= '[A-z0-9\s_-]{2,32}';
	public $regexp_ip		= '[0-9\.]+';
	public $regexp_coords	= '-?[0-9\.]+,\s*-?[0-9\.]+,\s*-?[0-9\.]+';
	
	private $appendable		= false;
	
	function parse_log_line($line){
		if(preg_match('/([0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2})\s+-\s+([^:]+):\s+([^:]+):\s+(.*?)$/i', $line, $matches)){
			$this->appendable = false;


			$time	= trim($matches[1]);
			$type	= trim($matches[2]);
			$more	= trim($matches[3]);
			$text	= trim($matches[4]);
			
			#echo '<pre>' . print_r($matches, true) . '</pre>';
			if($type=='TextLog' && $more=='INFO' && stripos($text, ' group joined')!==false){
				# 2016-06-23 20:15:55 - TextLog: INFO: aamche (192.168.1.14) from 'guest' group joined. (1/16)
			
				preg_match('/^(' . $this->regexp_name . ')\s+\((' . $this->regexp_ip .  ')\)/is', $text, $breakdown);
				
				#echo '<pre>' . print_r($matches, true) . '</pre>';
				#echo '<pre>' . print_r($breakdown, true) . '</pre>';exit;
			
				return array(
					'time'		=> $time,
					'type'		=> 'joined',
					'user'		=> trim($breakdown[1]),
					'ip'		=> trim($breakdown[2]),
					'text'		=> $text,
				);
			}

			# Join/Leave
			else if($type=='TextLog' && $more=='INFO' && stripos($text, ' disconnected.')!==false){
				preg_match('/(' . $this->regexp_name . ')\s+disconnected./is', $text, $breakdown);
				
				#echo '<pre>' . print_r($matches, true) . '</pre>';
				#echo '<pre>' . print_r($breakdown, true) . '</pre>';
			
				return array(
					'time'		=> $time,
					'type'		=> 'left',
					'user'		=> trim($breakdown[1]),
					'text'		=> $text,
				);
			}

			# Chat
			else if($type=='Utils' && $more=='INFO' && stripos($text, 'Broadcast:')!==false){
				#2016-06-25 22:20:41 - Utils: INFO: Broadcast: William: Anyone on?
				preg_match('/Broadcast:\s+(' . $this->regexp_name . '):\s+(.*?)$/is', $text, $breakdown);
				
				if(count($breakdown)>0){
					#echo '<pre style="color: green;">' . print_r($breakdown, true) . '</pre>';
					return array(
						'time'		=> $time,
						'type'		=> 'chat',
						'user'		=> $breakdown[1],
						'text'		=> $breakdown[2],
					);
				}
			} 
			
			# Server Command
			# [00:02:59] [Server thread/INFO]: [VanJeans: Teleported VanJeans to CountBarkula]
			else  if(false && $type=='server thread/info' && preg_match('/\[([A-z0-9_]+):\s+(.*?)]$/is', $text, $breakdown)){
				return array(
					'time'		=> $time,
					'type'		=> 'command',
					'user'		=> $breakdown[1],
					'text'		=> $breakdown[2],
				);
			} 
			
			# Error
			else  if(false && $type=='server thread/error'){
				$this->appendable = true;
				return array(
					'time'			=> $time,
					'type'			=> 'error',
					'text'			=> $text,
					'appendable'	=> true,
				);
			}

			# Unknown
			#return 'unknown: ' . print_r($matches, true);
		}
		// Append Java Error
		else if ($this->appendable===true){
			return array(
				'type'		=> 'append',
				'text'		=> $line,
			);
		}
		// Unknown line format
		else {
			#return 'line: ' . $line;
		}
		
		return false;
	}
	
	
	/* ============================================================
	 * EXPORT END
	 * ------------------------------------------------------------
	 * Exporter Function to call at end of export
	 */
	function export_end($server=null){
		if($server!==null){$this->server = $server;}
		
		// Add code here
	}
}

