/* ============================================================
 * App Specific Handlebar Functions
 */
function dateGetDay(str){
	var day;
	if (typeof(str)=='string' || (str>0)){
		day = new Date(str);
	} else {
		day = new Date();
	}
	day.setHours(0);
	day.setMinutes(0);
	day.setSeconds(0);
	day = Math.floor(day.getTime()/1000);
	return day;
}

Handlebars.registerHelper('datetimeOffset', function(str, options) {
	if(str==-1){
		var now = new Date();
		now = now.getTime()/1000;
		str = now - dateGetDay();
	} else if(typeof(str)=='undefined' || str===null){
		var now = new Date();
		now = now.getTime()/1000;
		str = now - dateGetDay(str);
	} else if (typeof(str)=='string' || !(str>0)){
		var now = new Date(str);
		now = now.getTime()/1000;
		str = now - dateGetDay(str);
	}
	if(str>0){
		return Math.round((str/86400)*1000000)/10000;
	}
});

Handlebars.registerHelper('datetimeWidth', function(str1, str2, options) {
	if(typeof(str1)=='undefined' || str1===null){
		str1 = dateGetDay();
	} else if (typeof(str1)=='string' || (str1>0)){
		str1 = new Date(str1);
		str1 = str1.getTime()/1000;
	}
	if(typeof(str2)=='undefined' || str2===null){
		var now = new Date(str1*1000);
		if(dateGetDay(str1)!=dateGetDay()){
			now.setHours(23);
			now.setMinutes(59);
			now.setSeconds(59);
		}
		str2 = now.getTime()/1000;
	} else if (typeof(str)=='string' || (str2>0)){
		str2 = new Date(str2);
		str2 = str2.getTime()/1000;
	}
	
	if(str1 < str2){
		return Math.round(((str2-str1)/86400)*1000000)/10000;
	}
});


Handlebars.registerHelper('locationsMapPoint', function(map, name, obj) {
	str = '<div class="location-user" data-user="' + name + '"';
	str += ' style="';
	
	var x = obj.coords.x;
	var y = obj.coords.y;
	var z = obj.coords.z;


	var w = h = 100;

	// Workout overall width
	if(map.dx_min<map.dx_max){
		w = Math.abs(map.dx_min) + parseInt(map.dx_max);
	} // todo

	// Workout overall height
	if(map.dz_min<map.dz_max){
		h = Math.abs(map.dz_min) + parseInt(map.dz_max);
	} // todo

	// Left
	str += 'left: ';
	if(x==map.dx_min){
		str += '0';
	} else {
		str += (((Math.abs(map.dx_min)+x)/w)*100) + '%';
	}
	str += ';';

	// Top
	str += 'top: ';
	if(y==map.dy_min){
		str += '0';
	} else {
		str += (((Math.abs(map.dz_min)+z)/h)*100) + '%';
	}
	str += ';';

	// Text Data
	str += '" data-toggle="tooltip" title="' + name + '<br />';
		if(obj.login){ 	str += 'login: ' + Handlebars.helpers.formatDate('H:i', obj.login) + '<br />';}
		if(obj.logoff){ str += 'logoff: ' + Handlebars.helpers.formatDate('H:i', obj.logoff) + '<br />';}
		if(obj.ip){ 	str += 'ip: ' + obj.ip + '<br />';}
		if(obj.coords.dimension){ str += 'dimension: ' + obj.coords.dimension + '<br />';}
		if(obj.coords){ str += 'coords: ' + obj.coords.x + ', ' + obj.coords.y + ', ' + obj.coords.z + '<br />';}

	str += '"></div>';

	// Done
	return new Handlebars.SafeString(str);
});


/* ============================================================
 * Application Pages and View objects
 */

/*
 * Config Object
 */
config = {
	model: {}
};

/*
 * Session Object
 */
session = {
	lastparse: null
};

/*
 * Servers Object
 */
servers = {
	model: {},
	collection: [],
	users_online_count: 0,
	users_online_names: [],
	nav_selector:function() {
		x = '';
		if($('#navbar-main .active').hasClass('view-either') || template.segment(1)==0){
			x += '<option value="0"';
			if(template.segment(1) == 0){
				x += ' selected="selected"';
			}
			x += '>All</option>';
		}
		$.each(this.collection, function( index, server ){
			if(typeof(server.capabilities)=='undefined' || server.capabilities.length>0){
				x += '<option value="' + server.id + '"';
				if(template.segment(1) == server.id){
					x += ' selected="selected"';
				}
				x += '>' + server.name + '</option>';
			}
		});
		$('#server-select').html(x);
	}
};

/* ============================================================
 * Template System Object
 */
template = {
	lastSegments: [],
	beforeLastSegments: [],
	views: {}
};

template.message = function(d, status){
		// Message
		if(d!==null && typeof(d.message)!='undefined'){
			html = '<div class="alert alert-';
			if(typeof(d.result)!='undefined' && d.result=='error'){
				html += 'danger';
			} else if(typeof(d.result)!='undefined' && d.result=='success'){
				html += 'success';
			} else {
				html += 'info';
			}
			html += '" role="alert">';
			if(typeof(d.message)!='undefined'){
				html += d.message;
			}
			html += '</div>';
			$("#main-alert").html(html).show();
		} else {
			$('#main-alert').hide().text('');
		}
	}

template.url = function(){
		return window.location.hash.replace('#', '');
	}

template.segments = function(url){
		url = url || template.url();
		return url.replace(/^#/ig, '').replace(/\?.*$/ig, '').split('/');
	}

template.segment = function(index, url, defaultVar){
		index		= index || 0;
		url			= url || window.location.hash;
		defaultVar	= defaultVar || '';

		var segs = template.segments(url);
		if(typeof(segs[index])!='undefined' && segs[index]!==null){
			return segs[index];
		}

		return defaultVar;
	}


template.segmentFilter = function(filter, url, defaultVar){
		filter		= filter || 0;
		url			= url || window.location.hash;
		defaultVar	= defaultVar || '';

		var a, b, c;

		// Trim off the left
		if(url.indexOf('/' + filter + ':')>=0){
			filter = '/' + filter + ':';
			a = url.indexOf(filter);
			b = url.substr(a + filter.length);
		} else if(url.indexOf(filter + ':')>=0){
			filter = filter + ':';
			a = url.indexOf(filter);
			b = url.substr(a + filter.length);
		} else {
			return defaultVar;
		}

		// Trim off the right
		if(b.indexOf('/')>=0){
			c = a.indexOf('/');
			b = url.substr(0, c);
		}
		if(b.indexOf(';')>=0){
			c = url.indexOf(';');
			b = url.substr(0, c);
		}

		return b;
	}

template.link = function(seg0, seg1, seg2){
	if(seg0=='chat' || seg0=='activity' || seg0=='maps'){
		seg1 = seg1	|| $('#server-select').val();
		seg2 = seg2 || '';

		var s = [];
		s.push(seg0);

		if(seg0=='chat' && $('#server-select:visible').length==0){
			seg1 = 0;
		}
		s.push(seg1);


		var thisDate = dateHelpers.getToday();
		if(seg2!=null && seg2!=''){
			thisDate = dateHelpers.strtotime(seg2);
		}

		if(dateHelpers.getYmd(thisDate)!=dateHelpers.getYmd()){
			s.push(seg2);
		}

		return '#' + s.join('/');
	} else if(seg0=='server'){
		seg1 = seg1 || 0;

		var s = [];
		s.push(seg0);
		if(seg1===null || seg1==''){
			seg1 = $('#server-select').val();
		}
		s.push(seg1);
		s.push(seg2);
		return '#' + s.join('/');
	} else {
		seg1 = seg1 || 0;

		var s = [];
		s.push(seg0);
		if(seg1===null || seg1==''){
			seg1 = $('#server-select').val();
		}
		s.push(seg1);
		return '#' + s.join('/');
	}
}

template.loading = function(url, id){
		id = id || 'overview';
		url = url || template.url();

		var selector = '#page-' + id, x = '';
		segments = template.segments(url);
		//alert(template.lastSegments[0] + ' == ' + segments[0]);

		// Some page fades
		if(	segments.length>=2
				&& template.lastSegments.length>=2
				&& template.lastSegments[0]==segments[0]
		){	
			$('#page-loading:hidden').fadeIn();
		} else if(	
				(segments[0]=='overview' && template.lastSegments[0]==segments[0])
			||	(segments[0]=='' && template.lastSegments[0]==segments[0])
		){	
			$('#page-loading:hidden').fadeIn();
		} else {
			//$('#page-active').html('');
			$('#page-loading:hidden').fadeIn();
		}
		
		// Highlight Navigation
		$('#page-active  .page-title').text('Loading...');
		$(".navbar:first-child ul.navbar-nav li").removeClass('active');
		if($(".navbar:first-child ul.navbar-nav li a[href^=#" + segments[0] + "]").length>1){
			$(".navbar:first-child ul.navbar-nav li a[href^='#" + segments[0] + "\/"  + segments[2] + "']").closest('.dropdown').addClass('active');
			$(".navbar:first-child ul.navbar-nav li a[href^='#" + segments[0] + "\/"  + segments[2] + "']").parent().addClass('active');
		} else {
			$(".navbar:first-child ul.navbar-nav li a[href^=#" + segments[0] + "]").parent().addClass('active');
		}
	};

template.loaded = function(){
		segments = template.segments();
		if(segments[0]==''){segments[0] = 'overview';}
		if(segments.length>=2 && parseInt(segments[1])>0){
			server_id = parseInt(segments[1]);
		} else {
			server_id = null;
		}

		var segment_2_isdate = (segments[0]=='chat' || segments[0]=='activity' || segments[0]=='maps' || segments[0]=='logs');

		// Update Nav Menu
		var dateSegment = '';
		var thisDate = dateHelpers.getToday();
		if(segment_2_isdate){
			if(segments.length>2 && segments[2]!=''){
				thisDate = dateHelpers.strtotime(segments[2]);
			}
		}

		var navbar = $('#navbar-collapse-1 ul.navbar-nav');

		// Determine view type
		var mode = 'overview';
		var mode_either = false;
		$("#navbar-collapse-1 ul.navbar-nav li a").each(function(){
			segs = template.segments($(this).attr('href'));
			if(segments[0]==segs[0]){
				if($(this).parent().hasClass('view-overview') || ($(this).parent().hasClass('view-either') && server_id===null)){
					mode = 'overview';
				} else if($(this).parent().hasClass('view-single') || ($(this).parent().hasClass('view-either') && server_id>0)){
					mode = 'single';
				}
				mode_either = $(this).parent().hasClass('view-either');
			};
		});

		// Update Nav: Update Link Visibility
		selector = '.navbar:first-child ul.navbar-nav li a.link-activity .badge';
		if(mode=='single'){
			$(navbar).find('li.view-overview').not('.view-always').hide();
			$(navbar).find('li.view-either').show();
			$(navbar).find('li.view-single').show();
			$(selector).show();
		} else {
			$(navbar).find('li.view-overview').show();
			$(navbar).find('li.view-either').show();
			$(navbar).find('li.view-single').not('.view-always').hide();
			$(selector).hide();
		}
		
		if(mode_either || mode=='single'){
			$('#server-select').show();
		} else {
			$('#server-select').hide();
		}
		if(mode_either){
			$('#server-select').attr('data-allow-all', true);
		} else {
			$('#server-select').attr('data-allow-all', false);
		}
		
		$(navbar).find('li.view-always').show();

		// Update Nav: Update Links
		if(mode=='single'){
			$('#server-select').val(server_id);
			$(navbar).find('li.view-single a, li.view-either a').each(function(){
				segs = template.segments($(this).attr('href'));
				if($(this).hasClass('has-date')){
					$(this).attr('href', template.link(segs[0], server_id, dateHelpers.getYmd(thisDate)));
				} else if(segs.length>=3){
					$(this).attr('href', template.link(segs[0], server_id, segs[2]));
				} else {
					$(this).attr('href', template.link(segs[0], server_id));
				}
			});
		} else {
			$(navbar).find('li.view-either a').each(function(){
				segs = template.segments($(this).attr('href'));
				$(this).attr('href', '#' + segs[0]);
			});
		}

		// Navigation
		if(segment_2_isdate){

			// Next Nav Date
			var nextDate = new Date(thisDate);
			nextDate.setDate(nextDate.getDate()+1);
			/*if(dateHelpers.getYmd(thisDate)==dateHelpers.getYmd()){
				$('#page-active .next').addClass('disabled');
				$('#page-active .next a').attr('href', '#' + segments[0] + '')
				$('#page-active .pager li:nth-child(2)').text('latest.log');
			} else {*/
				$('#page-active .next').removeClass('disabled');
				$('#page-active .next a').attr('href', template.link(segments[0], null, dateHelpers.getYmd(nextDate)));
				$('#page-active .pager li:nth-child(2)').text(dateHelpers.getYmd(thisDate));
			//}

			// Prev Nav Date
			var prevDate = new Date(thisDate);
			prevDate.setDate(prevDate.getDate()-1);
			$('#page-active .previous').removeClass('disabled');
			$('#page-active .previous a').attr('href', template.link(segments[0], null, dateHelpers.getYmd(prevDate)));
		}

		// Update Loading
		$('#page-active').removeClass('loading');
		$('#page-loading').stop().fadeOut();

		template.beforeLastSegments = template.lastSegments;
		template.lastSegments = template.segments();
	}


/* ============================================================
 * Router
 * http://krasimirtsonev.com/blog/article/A-modern-JavaScript-router-in-100-lines-history-api-pushState-hash-url
 * https://github.com/krasimir/navigo/
 * The MIT License (MIT)
 */
var Router = {
    routes: [],
    mode: null,
    root: '/',
    config: function(options) {
        this.mode = options && options.mode && options.mode == 'history' 
                    && !!(history.pushState) ? 'history' : 'hash';
        this.root = options && options.root ? '/' + this.clearSlashes(options.root) + '/' : '/';
        return this;
    },
    getFragment: function() {
        var fragment = '';
        if(this.mode === 'history') {
            fragment = this.clearSlashes(decodeURI(location.pathname + location.search));
            fragment = fragment.replace(/\?(.*)$/, '');
            fragment = this.root != '/' ? fragment.replace(this.root, '') : fragment;
        } else {
            var match = window.location.href.match(/#(.*)$/);
            fragment = match ? match[1] : '';
        }
        return this.clearSlashes(fragment);
    },
    clearSlashes: function(path) {
        return path.toString().replace(/\/$/, '').replace(/^\//, '');
    },
    add: function(re, handler) {
        if(typeof re == 'function') {
            handler = re;
            re = '';
        }
        this.routes.push({ re: re, handler: handler});
        return this;
    },
    remove: function(param) {
        for(var i=0, r; i<this.routes.length, r = this.routes[i]; i++) {
            if(r.handler === param || r.re.toString() === param.toString()) {
                this.routes.splice(i, 1); 
                return this;
            }
        }
        return this;
    },
    flush: function() {
        this.routes = [];
        this.mode = null;
        this.root = '/';
        return this;
    },
    check: function(f) {
        var fragment = f || this.getFragment();
        for(var i=0; i<this.routes.length; i++) {
            var match = fragment.match(this.routes[i].re);
            if(match) {
                match.shift();
                this.routes[i].handler.apply({}, match);
                return this;
            }           
        }
        return this;
    },
    listen: function() {
        var self = this;
        var current = self.getFragment();
        var fn = function() {
            if(current !== self.getFragment()) {
                current = self.getFragment();
                self.check(current);
            }
        }
        clearInterval(this.interval);
        this.interval = setInterval(fn, 50);
        return this;
    },
    navigate: function(path) {
        path = path ? path : '';
        if(this.mode === 'history') {
            history.pushState(null, null, this.root + this.clearSlashes(path));
        } else {
			window.location.href = window.location.href.replace(/#(.*)$/, '') + '#' + path;
        }
        return this;
    }
};

// configuration
Router.config({ 
	mode: 'hash', 
	root: ROOTURL
});

// returning the user to the initial state
//Router.navigate();

// adding routes
/* 
.add(/about/, function() {
    console.log('about');
})
.add(/products\/(.*)\/edit\/(.*)/, function() {
    console.log('products', arguments);
})
 */
Router.add(/(.*)/, function(url) {
	var segments = url.split('/');
	page_uri	= segments[0]	|| 'servers';
	server_id	= segments[1]	|| 0;
	seg1		= segments[2]	|| '';
	seg2		= segments[3]	|| '';
	seg3		= segments[4]	|| '';
	
	var request_url = page_uri;
	if(server_id!='' && typeof(server_id)!='undefined'){request_url += '/' + server_id;}
	if(seg1!='' && typeof(seg1)!='undefined'){request_url += '/' + seg1;}
	if(seg2!='' && typeof(seg2)!='undefined'){request_url += '/' + seg2;}
	if(seg3!='' && typeof(seg3)!='undefined'){request_url += '/' + seg3;}

	template.loading(request_url);

	$.ajax({
		url: 'data.php?view=' + request_url,
		type: 'GET',
		dataType: 'json',
		success: function(d){
			set_data(d);
			
			var source = $('#template-' + page_uri).html();
			if(typeof(source)=='undefined'){
				source = $('#template-e404').html();
			}

			var tmpl = Handlebars.compile(source);
			
			var data = d;
			if(servers.collection.length>0){
				data.servers = servers.collection;
			}
			if(template.segment(1)!=0){
				data.server = {id: template.segment(1)};
			}
			
			data.lists = {};
			
			// View Specific
			// Todo: abstract
			if(page_uri=='chat'){
				data.lists.activities = [
						{name: 'Chat', uri: 'chat'},
						{name: 'Joined', uri: 'joined'},
						{name: 'Left', uri: 'left'},
						{name: 'Command', uri: 'command'},
						{name: 'Info', uri: 'info'},
						{name: 'Error', uri: 'error'}
					];
			}
			
			
			// Display View
			var html = tmpl(data);
			$("#page-active").html(html);

			template.loaded();
			doRenderJs();
		},
		error: function(jqXHR, textStatus, errorThrown){
			source = $('#template-e500').html();
			var tmpl = Handlebars.compile(source);
			var html = tmpl({
				text_status: textStatus,
				message: jqXHR.responseText
			});
			$("#page-active").html(html);
			template.loaded();
			doRenderJs();
		}
	});
});
//.check('/products/12/edit/22');


/* ============================================================
 * App Specific Functions
 */

/*
 * Process Ajax payload and load into global data objects
 */
function set_data(d){
	if(typeof(d.message)!='undefined'){
		template.message(d, 'danger');
	} else {
		template.message(null);
	}

	if(typeof(d.errors)!='undefined'){
		d.result = 'error';
		d.message = d.errors.join('<br />');
		template.message(d, 'danger');
		template.loaded();
	}

	if(typeof(d.config)!='undefined'){
		config.model = d.config;
	}

	if(typeof(d.lastparse)!='undefined'){
		session.lastparse = d.lastparse;
	}

	if(typeof(d.servers)!='undefined'){
		if(d.servers.length==1 && servers.collection.length>0){
			for(var i=0;i<servers.collection.length;i++){
				if(servers.collection[i].id==d.servers[0].id){
					servers.collection[i] = d.servers[0];
				}
			}
		} else {
			servers.collection = d.servers;
		}
	}

	// Server(s)
	var users_online_count = 0;
	var users_online_names = [];
	if(typeof(d.servers)!='undefined'){
		servers.nav_selector();
	}
	if(typeof(d.servers)!='undefined'){
		var found_online_count = false;
		$.each(servers.collection, function( index, server ) {
			var state = server.state;
			if(typeof(state)=='object' && typeof(state.users)!='undefined' && typeof(state.users.online)!='undefined'){
				users_online_count += parseInt(state.users.online);
				
				if(state.users.sample && state.users.sample.length>0){
					for(var i=0; i<state.users.sample.length;i++){
						users_online_names.push(state.users.sample[i].name);
					}
				}

				found_online_count = true;
				
				
			}
		});

		if(found_online_count){
			servers.users_online_count = users_online_count;
			servers.users_online_names = users_online_names;
		}
	}
}


/*
 * doRenderJs
 * @returns nothing
 */
function doRenderJs(){
	$('[data-toggle="tooltip"]').tooltip({html: true});
	$('[data-toggle="popover"]').popover({html: true});
	$('.table.pageme tbody').pageMe({pagerSelector:'.table-pagination',showPrevNext:true,hidePageNumbers:false,perPage:25});
	$('.table.pageme').show();
	
	$('a[data-action],button[data-action]').click(function(){
		var action = $(this).attr('data-action');
		var functionname = 'action_' + action.replace(/[^A-z0-9]+/ig, '_');
		eval('if(typeof(' + functionname + ')=="function"){' + functionname + '(this);} else {alert("function \'' + functionname + '\' is undefined");}');
	});
	
	// Update View with server information
	if(servers.collection){
		// Update and containers that have online player names
		if(servers.users_online_names.length>0){
			var links = [];
			for(var i=0;i<servers.users_online_names.length;i++){
				user = servers.users_online_names[i];
				links.push('<a class="label label-primary" href="#users/name/' + user +'">' + user + '</a>');
			}
			$('.display-users-online-names').html(links.join(' '));
			
		} else {
			$('.display-users-online-names').text('');
		}
		
		
		$(".navbar .badge.display-users-online-count").text(servers.users_online_count);
		document.title = '(' + servers.users_online_count + ') ' + 'myMineMin';
	}

	// Update server page
	if(template.segment(0)=='servers' && servers.collection){
		$.each(servers.collection, server_live_activity.render);
	}	
}

/* ============================================================
 * Server Live Activity Handler
 */
server_live_activity = {
	update: function(){
		// If a full pull as only one thing being updated
		if(servers.collection.length==0){
			$.ajax({
				url: 'data.php?view=servers',
				dataType: 'json',
				success: function(data){
					server_live_activity.update_callback(data);
					$(servers.collection).each(function(index, row){
						$.ajax({
							url: 'data.php?view=servers/' + row.id + '/state',
							dataType: 'json',
							success: server_live_activity.update_callback
						});
					});
				}
			});
		} else {
			$(servers.collection).each(function(index, row){
				$.ajax({
					url: 'data.php?view=servers/' + row.id + '/state',
					dataType: 'json',
					success: server_live_activity.update_callback
				});
			});
		}
	},
	update_callback: function(data){
		if(data.servers){
			var refresh_page = false;
			if(data.lastparse && session.lastparse<data.lastparse){
				refresh_page = true;
			}
			
			set_data(data);
			doRenderJs();

			if(refresh_page===true && template.segment(0)!='servers'){
				Router.check();
			}
		} else {
			template.message({
				result: 'error',
				message: 'Error loading server status'
			});
		}
	},
	render: function(index, server){
		var rowContainer = $('[data-server_id=' + server.id + ']');
		if(server.state){
			if(typeof(server.state.ping)!='undefined'){
				$(rowContainer).find('.servers-server-ping').text(server.state.ping);
			}
			$(rowContainer).find('.servers-server-ping').removeClass('loading');

			var playerStat = '';
			if(typeof(server.state.users)!='undefined'){
				// User Count
				playerStat += '<div class="text-';
				if(typeof(server.state.users.online)!='undefined'){
					if(server.state.users.online>0){
						playerStat += 'success';
					} else {
						playerStat += 'muted';
					}
				} else {
					playerStat += 'muted';
				}
					playerStat += '">';

				if(server.state.users.online!=''){
					playerStat += server.state.users.online;
				} else {
					playerStat += 0;
				}
				if(server.state.users.max>0){
					playerStat += '/' + server.state.users.max;
				}
				playerStat += '</div>';

				if(server.state.users.sample && server.state.users.sample.length>0){
					for(var i=0; i<server.state.users.sample.length;i++){
						var user = server.state.users.sample[i];
						playerStat += '<a class="label label-primary" href="#users/name/' + user.name + '">' + user.name + '</a> ';
					}
				}

				$(rowContainer).find('.servers-server-users-online').html(playerStat).removeClass('loading');
			}

			if(server.state.result && server.state.result=='error'){
				playerStat += '<div class="text-danger">' + server.state.message + '</div>';
			}

			$(rowContainer).find('.servers-server-users-online').html(playerStat).removeClass('loading');
		} else if(server.errors){
			if(typeof(server.errors)=='array'){
				playerStat = '<div class="text-danger">' + server.errors.join('<br />') + '</div>';
			} else {
				playerStat = '<div class="text-danger">' + server.errors + '</div>';
			}

			$(rowContainer).find('.servers-server-ping').text('').removeClass('loading');
			$(rowContainer).find('.servers-server-users-online').html(playerStat).removeClass('loading');
		}
	}
}

function refresh_loader(){
	server_live_activity.update();
	//Router.check();
}


/* ============================================================
 * App Actions
 */
function action_checkversion(){
	$('.action-checkversion-result').html('<div class="panel panel-default"><div class="panel-heading">Checking for Updates</div><div class="panel-body">Querying server ... </div></div>');
	
	$.ajax({
		url: 'data.php?view=about&action=checkversion',
		dataType: 'json',
		success: function(data){
			var c1 = '.action-checkversion-result';
			
			if(data.result && data.result=='success'){
				$(c1).removeClass('alert alert-danger alert-info alert-success');
				var text = [];
				if(data.build){
					if(data.build==data.this_build){
						text.push('<div class="panel panel-success">');
						text.push('<div class="panel-heading">You are up to date.</div>');
						if(data.build){
							text.push('<div class="panel-body">Current Build: #' + data.build + '</div>');
						}
						text.push = '</div></div>';
					} else {
						text.push('<div class="panel panel-warning">');
						text.push('<div class="panel-heading">An update is available.</div>');
						text.push('<div class="panel-body">');
						if(data.build){
							text.push('<div>Current Build: #' + data.this_build + '</div>');
							text.push('<div>Available Build: #' + data.build + '</div>');
						}
						if(data.link){
							text.push('<div><a href="' + data.link + '" target="_blank">' + data.link + '</a></div>');
						}

						if(data.notes && data.build!=data.this_build){
							var notes = data.notes.replace(/\r\n/ig, '<br>');
							notes = notes.replace(/\r/ig, '<br>');
							notes = notes.replace(/\n/ig, '<br>');
							text.push('<div><br /><strong>Release Notes:</strong></div><div>' + notes + '</div>');
						}
						text.push = '</div></div>';
					}
				}
				
				$(c1).html(text.join(''));
			} else {
				if(data.message){
					$(c1).text(data.message);
				} else {
					$(c1).text('Error: unknown?');
				}
			}
		}
	});
}

/* ============================================================
 * Initilise Page
 */
$(document).ready(function(){
	$('#server-select').change(function(){
		var s = template.segments();
		
		if(s[0]==''){
			s[0] = 'overview';
		}
		s[1] = $(this).val();
		Router.navigate(s.join('/'));
	});
	
	var hash = window.location.hash;
	if(hash.length>2 && hash.substr(1)!='servers'){
		Router.navigate(hash.substr(1));
		//Router.check();
		refresh_loader();
	} else {
		Router.navigate('servers');
		Router.check();
	}
	Router.listen();
	
	setInterval(refresh_loader, 1000*60*5);
});


